@@ -11,6 +11,7 @@ from ... import array as da
 from ...base import tokenize
 from ...dataframe.core import new_dd_object
 from ...delayed import delayed
+from ...highlevelgraph import HighLevelGraph
 from ...utils import M, ensure_dict
 from ..core import DataFrame, Index, Series, has_parallel_type, new_dd_object
 from ..shuffle import set_partition
@@ -44,7 +45,7 @@ def _meta_from_array(x, columns=None, index=None, meta=None):
             raise ValueError("For a struct dtype, columns must be a list.")
         elif not all(i in x.dtype.names for i in columns):
             extra = sorted(set(columns).difference(x.dtype.names))
-            raise ValueError("dtype {0} doesn't have fields {1}".format(x.dtype, extra))
+            raise ValueError(f"dtype {x.dtype} doesn't have fields {extra}")
         fields = x.dtype.fields
         dtypes = [fields[n][0] if n in fields else "f8" for n in columns]
     elif x.ndim == 1:
@@ -66,9 +67,8 @@ def _meta_from_array(x, columns=None, index=None, meta=None):
             columns = list(range(x.shape[1])) if x.ndim == 2 else [0]
         elif len(columns) != x.shape[1]:
             raise ValueError(
-                "Number of column names must match width of the "
-                "array. Got {0} names for {1} "
-                "columns".format(len(columns), x.shape[1])
+                "Number of column names must match width of the array. "
+                f"Got {len(columns)} names for {x.shape[1]} columns"
             )
         dtypes = [x.dtype] * len(columns)
 
@@ -295,20 +295,17 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock, **kwar
         )
     new_name = "from_bcolz-" + token
 
-    dsk = dict(
-        (
-            (new_name, i),
-            (
-                dataframe_from_ctable,
-                x,
-                (slice(i * chunksize, (i + 1) * chunksize),),
-                columns,
-                categories,
-                lock,
-            ),
+    dsk = {
+        (new_name, i): (
+            dataframe_from_ctable,
+            x,
+            (slice(i * chunksize, (i + 1) * chunksize),),
+            columns,
+            categories,
+            lock,
         )
         for i in range(0, int(ceil(len(x) / chunksize)))
-    )
+    }
 
     meta = dataframe_from_ctable(x, slice(0, 0), columns, categories, lock)
     result = DataFrame(dsk, new_name, meta, divisions)
@@ -538,10 +535,10 @@ def to_bag(df, index=False, format="tuple"):
     if not isinstance(df, (DataFrame, Series)):
         raise TypeError("df must be either DataFrame or Series")
     name = "to_bag-" + tokenize(df, index, format)
-    dsk = dict(
-        ((name, i), (_df_to_bag, block, index, format))
+    dsk = {
+        (name, i): (_df_to_bag, block, index, format)
         for (i, block) in enumerate(df.__dask_keys__())
-    )
+    }
     dsk.update(df.__dask_optimize__(df.__dask_graph__(), df.__dask_keys__()))
     return Bag(dsk, name, df.npartitions)
 
@@ -573,10 +570,11 @@ def from_delayed(
 
     Parameters
     ----------
-    dfs : list of Delayed
+    dfs : list of Delayed or Future
         An iterable of ``dask.delayed.Delayed`` objects, such as come from
-        ``dask.delayed`` These comprise the individual partitions of the
-        resulting dataframe.
+        ``dask.delayed`` or an iterable of ``distributed.Future`` objects,
+        such as come from ``client.submit`` interface. These comprise the individual
+        partitions of the resulting dataframe.
     $META
     divisions : tuple, str, optional
         Partition boundaries along the index.
@@ -611,7 +609,7 @@ def from_delayed(
         dfs = [delayed(make_meta)(meta)]
 
     name = prefix + "-" + tokenize(*dfs)
-    dsk = merge(df.dask for df in dfs)
+    dsk = {}
     if verify_meta:
         for (i, df) in enumerate(dfs):
             dsk[(name, i)] = (check_meta, df.key, meta, "from_delayed")
@@ -626,7 +624,9 @@ def from_delayed(
         if len(divs) != len(dfs) + 1:
             raise ValueError("divisions should be a tuple of len(dfs) + 1")
 
-    df = new_dd_object(dsk, name, meta, divs)
+    df = new_dd_object(
+        HighLevelGraph.from_collections(name, dsk, dfs), name, meta, divs
+    )
 
     if divisions == "sorted":
         from ..shuffle import compute_and_set_divisions
