@@ -6,7 +6,7 @@ import pandas as pd
 import pytest
 
 import dask.dataframe as dd
-from dask.dataframe._compat import PANDAS_GT_100, PANDAS_GT_120, PANDAS_VERSION
+from dask.dataframe._compat import PANDAS_GT_120, PANDAS_VERSION
 from dask.dataframe.utils import assert_dask_graph, assert_eq, make_meta
 
 try:
@@ -134,9 +134,9 @@ def test_deterministic_arithmetic_names():
     df = pd.DataFrame({"x": [1, 2, 3, 4], "y": [5, 6, 7, 8]})
     a = dd.from_pandas(df, npartitions=2)
 
-    assert sorted((a.x + a.y ** 2).dask) == sorted((a.x + a.y ** 2).dask)
-    assert sorted((a.x + a.y ** 2).dask) != sorted((a.x + a.y ** 3).dask)
-    assert sorted((a.x + a.y ** 2).dask) != sorted((a.x - a.y ** 2).dask)
+    assert sorted((a.x + a.y**2).dask) == sorted((a.x + a.y**2).dask)
+    assert sorted((a.x + a.y**2).dask) != sorted((a.x + a.y**3).dask)
+    assert sorted((a.x + a.y**2).dask) != sorted((a.x - a.y**2).dask)
 
 
 @pytest.mark.slow
@@ -277,7 +277,7 @@ def check_series_arithmetics(l, r, el, er, allow_comparison_ops=True):
     assert_eq(l - r, el - er)
     assert_eq(l / r, el / er)
     assert_eq(l // r, el // er)
-    assert_eq(l ** r, el ** er)
+    assert_eq(l**r, el**er)
     assert_eq(l % r, el % er)
 
     if allow_comparison_ops:
@@ -306,7 +306,7 @@ def check_series_arithmetics(l, r, el, er, allow_comparison_ops=True):
     assert_eq(l | True, el | True)
     assert_eq(l ^ True, el ^ True)
     assert_eq(l // 2, el // 2)
-    assert_eq(l ** 2, el ** 2)
+    assert_eq(l**2, el**2)
     assert_eq(l % 2, el % 2)
     assert_eq(l > 2, el > 2)
     assert_eq(l < 2, el < 2)
@@ -323,7 +323,7 @@ def check_series_arithmetics(l, r, el, er, allow_comparison_ops=True):
     assert_eq(True | r, True | er)
     assert_eq(True ^ r, True ^ er)
     assert_eq(2 // r, 2 // er)
-    assert_eq(2 ** r, 2 ** er)
+    assert_eq(2**r, 2**er)
     assert_eq(2 % r, 2 % er)
     assert_eq(2 > r, 2 > er)
     assert_eq(2 < r, 2 < er)
@@ -361,7 +361,7 @@ def check_frame_arithmetics(l, r, el, er, allow_comparison_ops=True):
     assert_eq(l - r, el - er)
     assert_eq(l / r, el / er)
     assert_eq(l // r, el // er)
-    assert_eq(l ** r, el ** er)
+    assert_eq(l**r, el**er)
     assert_eq(l % r, el % er)
 
     if allow_comparison_ops:
@@ -390,7 +390,7 @@ def check_frame_arithmetics(l, r, el, er, allow_comparison_ops=True):
     assert_eq(l | True, el | True)
     assert_eq(l ^ True, el ^ True)
     assert_eq(l // 2, el // 2)
-    assert_eq(l ** 2, el ** 2)
+    assert_eq(l**2, el**2)
     assert_eq(l % 2, el % 2)
     assert_eq(l > 2, el > 2)
     assert_eq(l < 2, el < 2)
@@ -407,7 +407,7 @@ def check_frame_arithmetics(l, r, el, er, allow_comparison_ops=True):
     assert_eq(True | l, True | el)
     assert_eq(True ^ l, True ^ el)
     assert_eq(2 // l, 2 // el)
-    assert_eq(2 ** l, 2 ** el)
+    assert_eq(2**l, 2**el)
     assert_eq(2 % l, 2 % el)
     assert_eq(2 > l, 2 > el)
     assert_eq(2 < l, 2 < el)
@@ -448,7 +448,7 @@ def test_scalar_arithmetics():
     assert_eq(l - r, el - er)
     assert_eq(l / r, el / er)
     assert_eq(l // r, el // er)
-    assert_eq(l ** r, el ** er)
+    assert_eq(l**r, el**er)
     assert_eq(l % r, el % er)
 
     assert_eq(l & r, el & er)
@@ -469,7 +469,7 @@ def test_scalar_arithmetics():
     assert_eq(l | True, el | True)
     assert_eq(l ^ True, el ^ True)
     assert_eq(l // 2, el // 2)
-    assert_eq(l ** 2, el ** 2)
+    assert_eq(l**2, el**2)
     assert_eq(l % 2, el % 2)
     assert_eq(l > 2, el > 2)
     assert_eq(l < 2, el < 2)
@@ -486,7 +486,7 @@ def test_scalar_arithmetics():
     assert_eq(True | r, True | er)
     assert_eq(True ^ r, True ^ er)
     assert_eq(2 // r, 2 // er)
-    assert_eq(2 ** r, 2 ** er)
+    assert_eq(2**r, 2**er)
     assert_eq(2 % r, 2 % er)
     assert_eq(2 > r, 2 > er)
     assert_eq(2 < r, 2 < er)
@@ -1173,16 +1173,11 @@ def test_reductions_frame_dtypes():
     df_no_timedelta = df.drop("timedelta", axis=1, inplace=False)
     ddf_no_timedelta = dd.from_pandas(df_no_timedelta, 3)
 
-    if not PANDAS_GT_100:
-        # https://github.com/pandas-dev/pandas/issues/30886
-        assert_eq(df.sum(), ddf.sum())
-        assert_eq(df_no_timedelta.mean(), ddf_no_timedelta.mean())
-    else:
-        assert_eq(df.drop(columns="dt").sum(), ddf.drop(columns="dt").sum())
-        assert_eq(
-            df_no_timedelta.drop(columns="dt").mean(),
-            ddf_no_timedelta.drop(columns="dt").mean(),
-        )
+    assert_eq(df.drop(columns="dt").sum(), ddf.drop(columns="dt").sum())
+    assert_eq(
+        df_no_timedelta.drop(columns="dt").mean(),
+        ddf_no_timedelta.drop(columns="dt").mean(),
+    )
 
     assert_eq(df.prod(), ddf.prod())
     assert_eq(df.product(), ddf.product())
@@ -1266,8 +1261,7 @@ def test_reductions_frame_dtypes_numeric_only():
         assert_eq(
             getattr(df, func)(**kwargs),
             getattr(ddf, func)(**kwargs),
-            check_dtypes=func in ["mean", "max"]
-            and (PANDAS_GT_120 or not PANDAS_GT_100),
+            check_dtype=func in ["mean", "max"] and PANDAS_GT_120,
         )
         with pytest.raises(NotImplementedError, match="'numeric_only=False"):
             getattr(ddf, func)(numeric_only=False)
@@ -1293,10 +1287,7 @@ def test_reductions_frame_dtypes_numeric_only():
         assert_eq(
             getattr(df_numerics, func)(),
             getattr(ddf_numerics, func)(),
-            check_dtypes=func in ["mean", "max"]
-            and (PANDAS_GT_120 or not PANDAS_GT_100),
-            check_dtype=func in ["mean", "max"]
-            and (PANDAS_GT_120 or not PANDAS_GT_100),
+            check_dtype=func in ["mean", "max"] and PANDAS_GT_120,
         )
 
 
