@@ -7,8 +7,6 @@ import os
 from functools import partial
 from operator import add
 
-from tornado import gen
-
 from distributed.utils_test import client as c  # noqa F401
 from distributed.utils_test import cluster_fixture  # noqa F401
 from distributed.utils_test import loop  # noqa F401
@@ -36,17 +34,17 @@ def test_can_import_nested_things():
 
 
 @gen_cluster(client=True)
-def test_persist(c, s, a, b):
+async def test_persist(c, s, a, b):
     x = delayed(inc)(1)
     (x2,) = persist(x)
 
-    yield wait(x2)
+    await wait(x2)
     assert x2.key in a.data or x2.key in b.data
 
     y = delayed(inc)(10)
     y2, one = persist(y, 1)
 
-    yield wait(y2)
+    await wait(y2)
     assert y2.key in a.data or y2.key in b.data
 
 
@@ -133,14 +131,14 @@ def test_futures_to_delayed_array(c):
 
 
 @gen_cluster(client=True)
-def test_local_get_with_distributed_active(c, s, a, b):
+async def test_local_get_with_distributed_active(c, s, a, b):
     with dask.config.set(scheduler="sync"):
         x = delayed(inc)(1).persist()
-    yield gen.sleep(0.01)
+    await asyncio.sleep(0.01)
     assert not s.tasks  # scheduler hasn't done anything
 
     x = delayed(inc)(2).persist(scheduler="sync")  # noqa F841
-    yield gen.sleep(0.01)
+    await asyncio.sleep(0.01)
     assert not s.tasks  # scheduler hasn't done anything
 
 
@@ -177,15 +175,21 @@ def test_to_hdf_scheduler_distributed(npartitions, c):
 
 
 @gen_cluster(client=True)
-def test_serializable_groupby_agg(c, s, a, b):
+async def test_serializable_groupby_agg(c, s, a, b):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     df = pd.DataFrame({"x": [1, 2, 3, 4], "y": [1, 0, 1, 0]})
     ddf = dd.from_pandas(df, npartitions=2)
 
-    result = ddf.groupby("y").agg("count")
+    result = ddf.groupby("y").agg("count", split_out=2)
 
-    yield c.compute(result)
+    # Check Culling and Compute
+    agg0 = await c.compute(result.partitions[0])
+    agg1 = await c.compute(result.partitions[1])
+    dd.utils.assert_eq(
+        pd.concat([agg0, agg1]),
+        pd.DataFrame({"x": [2, 2], "y": [0, 1]}).set_index("y"),
+    )
 
 
 def test_futures_in_graph(c):
@@ -217,9 +221,9 @@ def test_zarr_in_memory_distributed_err(c):
     da = pytest.importorskip("dask.array")
     zarr = pytest.importorskip("zarr")
 
-    c = (1, 1)
-    a = da.ones((3, 3), chunks=c)
-    z = zarr.zeros_like(a, chunks=c)
+    chunks = (1, 1)
+    a = da.ones((3, 3), chunks=chunks)
+    z = zarr.zeros_like(a, chunks=chunks)
 
     with pytest.raises(RuntimeError):
         a.to_zarr(z)
@@ -322,12 +326,19 @@ def test_blockwise_array_creation(c, io, fuse):
     ["parquet-pyarrow", "parquet-fastparquet", "csv", "hdf"],
 )
 @pytest.mark.parametrize("fuse", [True, False, None])
-def test_blockwise_dataframe_io(c, tmpdir, io, fuse):
+@pytest.mark.parametrize("from_futures", [True, False])
+def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
 
     df = pd.DataFrame({"x": [1, 2, 3] * 5, "y": range(15)})
-    ddf0 = dd.from_pandas(df, npartitions=3)
+
+    if from_futures:
+        parts = [df.iloc[:5], df.iloc[5:10], df.iloc[10:15]]
+        futs = c.scatter(parts)
+        ddf0 = dd.from_delayed(futs, meta=parts[0])
+    else:
+        ddf0 = dd.from_pandas(df, npartitions=3)
 
     if io.startswith("parquet"):
         if io == "parquet-pyarrow":
@@ -631,3 +642,20 @@ async def test_annotation_pack_unpack(c, s, a, b):
     unpacked_hlg = HighLevelGraph.__dask_distributed_unpack__(packed_hlg)
     annotations = unpacked_hlg["annotations"]
     assert annotations == {"workers": {"n": ("alice",)}}
+
+
+@gen_cluster(client=True)
+async def test_pack_MaterializedLayer_handles_futures_in_graph_properly(c, s, a, b):
+    fut = c.submit(inc, 1)
+
+    hlg = HighLevelGraph(
+        {"l1": MaterializedLayer({"x": fut, "y": (inc, "x"), "z": (inc, "y")})},
+        {"l1": set()},
+    )
+    # fill hlg.key_dependencies cache. This excludes known futures, so only
+    # includes a subset of all dependencies. Previously if the cache was present
+    # the future dependencies would be missing when packed.
+    hlg.get_all_dependencies()
+    packed = hlg.__dask_distributed_pack__(c, ["z"], {})
+    unpacked = HighLevelGraph.__dask_distributed_unpack__(packed)
+    assert unpacked["deps"] == {"x": {fut.key}, "y": {fut.key}, "z": {"y"}}
